.action_col {
    text-align: right;
}

.action_col a {
    color: black;
}

.error-header {
	white-space: pre;
}

.default_col {
    text-align: center;
}

.list_item > div input.flush-left {
    margin-left: 4px;
}

div.scrollable {
    overflow-y: auto;
    height: 180px;
}

div.big_scrollable {
    overflow-y: auto;
    height: 360px;
}

div.half_width {
    width: 48%;
    margin: 1%;
    display: inline-block;
}

/* Create two equal columns that floats next to each other */
div.column {
  float: left;
  width: 50%;
  padding: 10px;
  height: 300px; /* Should be removed. Only for demonstration */
}

input.small-input {
    height: 22px;
    width: 100%;
    padding: 2px 6px 0px 6px;
    font-size: 10px;
}

#jobs {
    width: 98%;
    margin: 1%;
}

.icon-button {
    padding-right: 8px;
}

label {
    margin-right: 8px;
    margin-bottom: 8px;
}

.job_form input {
    margin-right: 15px;
}


.form-style-2{
	/*max-width: 500px;*/
	padding: 20px 12px 10px 20px;
	font: 13px Arial, Helvetica, sans-serif;
}
.form-style-2-heading{
	font-weight: bold;
	font-style: italic;
	border-bottom: 2px solid #ddd;
	margin-bottom: 20px;
	font-size: 15px;
	padding-bottom: 3px;
}
.form-style-2 label{
	display: block;
	margin: 10px 0px 15px 0px;
}
.form-style-2 label > span{
	width: 100px;
	font-weight: bold;
	float: left;
	padding-top: 8px;
	padding-right: 5px;
}
.form-style-2 span.required{
	color:red;
}
.form-style-2 input.input-field, .form-style-2 .select-field{
	width: 48%;
}
.form-style-2 input.input-field,
.form-style-2 .short-input-field,
.form-style-2 .long-input-field,
.form-style-2 .textarea-field,
 .form-style-2 .select-field{
	box-sizing: border-box;
	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	border: 1px solid #C2C2C2;
	box-shadow: 1px 1px 4px #EBEBEB;
	-moz-box-shadow: 1px 1px 4px #EBEBEB;
	-webkit-box-shadow: 1px 1px 4px #EBEBEB;
	border-radius: 3px;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	padding: 7px;
	outline: none;
}
.form-style-2 .input-field:focus,
.form-style-2 .schedule:focus,
.form-style-2 short-textarea-field:focus,
.form-style-2 .textarea-field:focus,
.form-style-2 .select-field:focus{
	border: 1px solid #0C0;
}
.form-style-2 .textarea-field{
	height:100px;
	width: 75%;
}

.form-style-2 .short-textarea-field{
	height:50px;
	width: 75%;
}

.form-style-2 .short-input-field{
	width: 150px;
}
.form-style-2 .long-input-field{
	width: 75%;
}

.form-style-2 input[type=submit],
.form-style-2 input[type=button]{
	border: none;
	padding: 8px 15px 8px 15px;
	background: #FF8500;
	color: #fff;
	box-shadow: 1px 1px 4px #DADADA;
	-moz-box-shadow: 1px 1px 4px #DADADA;
	-webkit-box-shadow: 1px 1px 4px #DADADA;
	border-radius: 3px;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
}
.form-style-2 input[type=submit]:hover,
.form-style-2 input[type=button]:hover{
	background: #EA7B00;
	color: #fff;
}
